#include "fsm.h"
#include "../shmem.h"
#include <syslog.h>
#include <sys/types.h>
#include <sys/shm.h>

void fsm_init(int shared_memory_key) {
	if(!(fsm = (struct fsm *) malloc(sizeof(struct fsm)))) {
		syslog(LOG_ERR,"Unable to malloc fsm struct");
		fsm = NULL;
		return;
	}
	
	if((fsm->shmkey = shmget(shared_memory_key,sizeof(struct msg), 0666 | IPC_CREAT)) < 0) {
		free(fsm);
		fsm = NULL;
		syslog(LOG_ERR,"Unable to find or create shared memory to %d",shared_memory_key);
		return;
	}

	if((fsm->msg = (struct msg *) shmat(fsm->shmkey,(char *) 0, 0)) == (struct msg *) -1) {
		free(fsm);
		fsm = NULL;
		syslog(LOG_ERR,"Unable to bind shraed memory to %d",shared_memory_key);
		return;
	}
	
	return;
}

char *fsm_translate(char *file) {
	static char ret[256];
	struct msg *msg;

	int i;
	if(!file) return(file);
	if(!fsm) return(file);
	msg = fsm->msg;

	for(i=0;i<=msg->max_entry;i++) {
		if((msg->flags[i] & SHMEM_ACTIVE) && !strcmp(file,msg->to[i])) {
			strcpy(ret,msg->from[i]);
			if(!(msg->flags[i] & SHMEM_OPENED)) msg->flags[i] += SHMEM_OPENED;
			return(ret);
		}
	}

	return(file);
}

char *fsm_map(char *file) {
	static char ret[256];
	struct msg *msg;
	int i;
	if(!file) return(file);
	if(!fsm) return(file);
	msg = fsm->msg;

	for(i=0;i<=msg->max_entry;i++) {
		if((msg->flags[i] & SHMEM_ACTIVE) && !strcmp(file,msg->to[i])) {
			strcpy(ret,msg->from[i]);
			return(ret);
		}
	}
	
	return(file);
}

int fsm_notify(char *file) {
	struct msg *msg;
	int i;
	
	if(!file) return(0);		
	if(!fsm) return(0);
	
	msg = fsm->msg;

	for(i=0;i<=msg->max_entry;i++) {
		if((msg->flags[i] & SHMEM_ACTIVE) && !strcmp(file,msg->to[i])) {
			if(!(msg->flags[i] & SHMEM_TOUCHED)) msg->flags[i] += SHMEM_TOUCHED;
			return(1);
		}
	}
	
	return(0);
}
